"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.callbackHandler = exports.signinHandler = void 0;
const ts_serverless_authentication_1 = require("ts-serverless-authentication");
async function signinHandler(config, options) {
    const customGoogle = new ts_serverless_authentication_1.Provider(config);
    const signinOptions = options || {};
    signinOptions.signin_uri = 'https://accounts.google.com/o/oauth2/v2/auth';
    signinOptions.scope = 'profile email';
    signinOptions.response_type = 'code';
    return customGoogle.signin(signinOptions);
}
exports.signinHandler = signinHandler;
async function callbackHandler(event, config) {
    const customGoogle = new ts_serverless_authentication_1.Provider(config);
    const profileMap = (response) => {
        return new ts_serverless_authentication_1.Profile({
            id: response.id,
            name: response.displayName,
            email: response.emails ? response.emails[0].value : null,
            picture: response.image ? response.image.url : null,
            provider: 'custom-google',
            at: response.access_token
        });
    };
    const options = {
        authorization_uri: 'https://www.googleapis.com/oauth2/v4/token',
        profile_uri: 'https://www.googleapis.com/plus/v1/people/me',
        profileMap
    };
    const callbackResult = await customGoogle.callback(event, options, {
        authorization: { grant_type: 'authorization_code' }
    });
    return callbackResult;
}
exports.callbackHandler = callbackHandler;
//# sourceMappingURL=custom-google.js.map