"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.callbackHandler = void 0;
// Config
const ts_serverless_authentication_1 = require("ts-serverless-authentication");
const crypto = __importStar(require("crypto"));
// import * as customGoogle from '../custom-google';
// Common
const dynamoCache_1 = require("../storage/dynamo/dynamoCache");
const users = __importStar(require("../storage/usersStorage"));
const helpers_1 = require("../helpers");
function createUserId(data, secret) {
    const hmac = crypto.createHmac('sha256', secret);
    hmac.update(data);
    return hmac.digest('hex');
}
/**
 * Error response
 * @param error
 */
function errorResponse(error, providerConfig) {
    const { url } = ts_serverless_authentication_1.utils.errorResponse(error, providerConfig);
    return {
        statusCode: 302,
        headers: {
            Location: url
        }
    };
}
/**
 * Token response
 * @param data
 */
function tokenResponse(data, providerConfig) {
    const { url } = ts_serverless_authentication_1.utils.tokenResponse(data, providerConfig);
    return {
        statusCode: 302,
        headers: {
            Location: url
        }
    };
}
/**
 * Handles the response
 * @param error
 * @param profile
 * @param state
 * User Saved in this function.
 */
const handleResponse = async (profileState, providerConfig, cache) => {
    try {
        await cache.revokeState(profileState.state);
        if (!providerConfig.token_secret) {
            throw new Error('No token secret provided.');
        }
        const id = createUserId(`${profileState.profile.provider}-${profileState.profile.id}`, providerConfig.token_secret);
        const data = helpers_1.createResponseData(id, providerConfig);
        const userContext = await users.saveUser(Object.assign(Object.assign({}, profileState.profile), { userId: id }));
        // saveUser can optionally return an authorizer context map
        // see http://docs.aws.amazon.com/apigateway/latest/developerguide/use-custom-authorizer.html
        if (typeof userContext === 'object' && !Array.isArray(userContext)) {
            data.authorizationToken.payload = Object.assign(Object.assign({}, userContext), data.authorizationToken.payload);
        }
        const result = await cache.saveRefreshToken(id, data.authorizationToken.payload);
        return tokenResponse(Object.assign(Object.assign({}, data), { refreshToken: result }), providerConfig);
    }
    catch (exception) {
        return errorResponse({ error: exception }, providerConfig);
    }
};
/**
 * Callback Handler
 * @param proxyEvent
 * @param context
 */
async function callbackHandler(proxyEvent, providerCallbackHandler) {
    const event = {
        provider: proxyEvent.pathParameters.provider,
        stage: proxyEvent.requestContext.stage,
        host: proxyEvent.headers.Host,
        code: proxyEvent.queryStringParameters.code,
        state: proxyEvent.queryStringParameters.state
    };
    const providerConfig = ts_serverless_authentication_1.config(event);
    const response = await providerCallbackHandler(event, providerConfig);
    const cache = new dynamoCache_1.dynamoCache({ region: 'us-west-2', endpoint: 'http://localhost:8000' });
    if (!response) {
        throw new Error('Provider or response was not found.');
    }
    return handleResponse(response, providerConfig, cache);
}
exports.callbackHandler = callbackHandler;
//# sourceMappingURL=callbackHandler.js.map