"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshHandler = void 0;
// Config
const ts_serverless_authentication_1 = require("ts-serverless-authentication");
// Common
const dynamoCache_1 = require("../storage/dynamo/dynamoCache");
const helpers_1 = require("../helpers");
/**
 * Refresh Handler
 * @param event
 * @param callback
 */
async function refreshHandler(event) {
    const cache = new dynamoCache_1.dynamoCache({ region: 'us-west-2', endpoint: 'http://localhost:8000' });
    const refreshToken = event.refresh_token;
    // user refresh token to get userid & provider from cache table
    try {
        const results = await cache.revokeRefreshToken(refreshToken);
        const providerConfig = ts_serverless_authentication_1.config({ provider: '', stage: event.stage });
        const { id } = results;
        const data = Object.assign(Object.assign({}, helpers_1.createResponseData(id, providerConfig)), { refreshToken: results.token });
        if (typeof results.payload === 'object') {
            data.authorizationToken.payload = Object.assign(Object.assign({}, data.authorizationToken.payload), results.payload);
        }
        if (!providerConfig.token_secret) {
            throw new Error('No token secret provided.');
        }
        const authorization_token = ts_serverless_authentication_1.utils.createToken(data.authorizationToken.payload, providerConfig.token_secret, data.authorizationToken.options);
        const result = { authorization_token, refresh_token: data.refreshToken, id };
        return result;
    }
    catch (exception) {
        throw new Error(`refreshHandler: ${exception}`);
    }
}
exports.refreshHandler = refreshHandler;
//# sourceMappingURL=refreshHandler.js.map