"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = __importStar(require("aws-sdk"));
const cognitoIdentityServiceProvider = new AWS.CognitoIdentityServiceProvider();
const getUserAttributes = (profile) => {
    const attributes = [
        'address',
        'birthdate',
        'email',
        'family_name',
        'gender',
        'given_name',
        'locale',
        'middle_name',
        'name',
        'nickname',
        'phone_number',
        'picture',
        'preferred_username',
        'profile',
        'timezone',
        'website'
    ];
    return attributes.filter((key) => {
        return Object.prototype.hasOwnProperty.call(profile, key);
    }).map((v) => { return { Name: v, Value: profile[v] }; });
};
const saveUser = async (profile) => {
    const params = {
        UserPoolId: process.env.USER_POOL_ID || '',
        Username: profile.userId,
        DesiredDeliveryMediums: [
            'EMAIL'
        ],
        ForceAliasCreation: false,
        MessageAction: 'SUPPRESS',
        TemporaryPassword: 'tempPassword1!',
        UserAttributes: getUserAttributes(profile)
    };
    return await cognitoIdentityServiceProvider
        .adminCreateUser(params).promise();
};
const updateUser = async (profile) => {
    const params = {
        UserAttributes: getUserAttributes(profile),
        UserPoolId: process.env.USER_POOL_ID || '',
        Username: profile.userId
    };
    return await cognitoIdentityServiceProvider
        .adminUpdateUserAttributes(params).promise();
};
const saveOrUpdateUser = async (profile) => {
    const params = {
        UserPoolId: process.env.USER_POOL_ID || '',
        Username: profile.userId
    };
    return cognitoIdentityServiceProvider
        .adminGetUser(params).promise()
        .then(() => updateUser(profile))
        .catch((error) => {
        if (error.code === 'UserNotFoundException') {
            return saveUser(profile);
        }
        throw error;
    });
};
module.exports = {
    saveOrUpdateUser,
    saveUser,
    updateUser
};
//# sourceMappingURL=cognitoUser.js.map