"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dynamoCache = void 0;
const AWS = __importStar(require("aws-sdk"));
const Cacheable_1 = require("../abstract/Cacheable");
class dynamoCache extends Cacheable_1.Cacheable {
    constructor(config) {
        super();
        this.config = {
            region: process.env.REGION || 'eu-west-1'
        };
        /**
         * Creates OAuth State
         */
        this.createState = async () => {
            const state = this.hash();
            const params = {
                TableName: process.env.CACHE_DB_NAME || '',
                Item: {
                    token: state,
                    type: 'STATE',
                    expired: false
                }
            };
            try {
                await this.dynamodb.put(params).promise();
                return state;
                // .then(() => state)
            }
            catch (error) {
                throw new Error(`createState: ${error}`);
            }
        };
        /**
         * Revokes OAuth State
         * @param state
         */
        this.revokeState = async (state) => {
            const queryToken = async () => {
                const params = {
                    TableName: process.env.CACHE_DB_NAME || '',
                    ProjectionExpression: '#token, #type, Expired',
                    KeyConditionExpression: '#token = :token and #type = :type',
                    ExpressionAttributeNames: {
                        '#token': 'token',
                        '#type': 'type'
                    },
                    ExpressionAttributeValues: {
                        ':token': state,
                        ':type': 'STATE'
                    }
                };
                try {
                    const queryResults = await this.dynamodb.query(params).promise();
                    return queryResults;
                }
                catch (error) {
                    throw new Error(`revokeState - queryToken: ${error}`);
                }
            };
            const insertToken = async (data) => {
                const item = data.Items[0];
                if (item.expired) {
                    throw new Error('State expired');
                }
                else {
                    const params = {
                        TableName: process.env.CACHE_DB_NAME || '',
                        Item: {
                            token: state,
                            type: 'STATE',
                            expired: true
                        }
                    };
                    try {
                        await this.dynamodb.put(params).promise();
                        if (item.token) {
                            return item.token;
                        }
                        else {
                            throw new Error(`revokeState - No Token`);
                        }
                        // .then(() => item.token)
                    }
                    catch (error) {
                        throw new Error(`revokeState - insert token: ${error}`);
                    }
                }
            };
            const queriedToken = await queryToken();
            const insertedToken = await insertToken(queriedToken);
            if (state !== insertedToken) {
                throw new Error('State mismatch');
            }
            return insertedToken;
        };
        /**
         * Creates and saves refresh token
         * @param user
         */
        this.saveRefreshToken = async (user, payload) => {
            const token = this.hash();
            const params = {
                TableName: process.env.CACHE_DB_NAME || '',
                Item: {
                    token: token,
                    type: 'REFRESH',
                    expired: false,
                    userId: user,
                    payload: JSON.stringify(payload || {})
                }
            };
            try {
                await this.dynamodb.put(params).promise();
                return token;
            }
            catch (error) {
                throw new Error(`saveRefreshToken: ${error}`);
            }
        };
        /**
         * Revokes old refresh token and creates new
         * @param oldToken
         */
        this.revokeRefreshToken = async (oldToken) => {
            if (oldToken.match(/[A-Fa-f0-9]{64}/)) {
                const token = this.hash();
                const queryToken = async () => {
                    const params = {
                        TableName: process.env.CACHE_DB_NAME || '',
                        ProjectionExpression: '#token, #type, #userId',
                        KeyConditionExpression: '#token = :token and #type = :type',
                        ExpressionAttributeNames: {
                            '#token': 'token',
                            '#type': 'type',
                            '#userId': 'userId'
                        },
                        ExpressionAttributeValues: {
                            ':token': oldToken,
                            ':type': 'REFRESH'
                        }
                    };
                    try {
                        return await this.dynamodb.query(params).promise();
                    }
                    catch (error) {
                        throw new Error(`revokeRefreshToken - queryToken: ${error}`);
                    }
                };
                const newRefreshToken = async (data) => {
                    const { userId, payload } = data.Items[0];
                    const params = {
                        TableName: process.env.CACHE_DB_NAME || '',
                        Item: {
                            token,
                            type: 'REFRESH',
                            expired: false,
                            userId,
                            payload
                        }
                    };
                    try {
                        await this.dynamodb.put(params).promise();
                        return userId;
                    }
                    catch (error) {
                        throw new Error(`revokeRefreshToken - newRefreshToken: ${error}`);
                    }
                };
                const expireRefreshToken = async (userId) => {
                    const params = {
                        TableName: process.env.CACHE_DB_NAME || '',
                        Item: {
                            token: oldToken,
                            type: 'REFRESH',
                            expired: true,
                            userId: userId
                        }
                    };
                    try {
                        await this.dynamodb.put(params).promise();
                        return userId;
                    }
                    catch (error) {
                        throw new Error(`revokeRefreshToken - expireRefreshToken: ${error}`);
                    }
                };
                const queriedToken = await queryToken();
                const newToken = await newRefreshToken(queriedToken);
                const userId = await expireRefreshToken(newToken);
                return {
                    id: userId,
                    token: token,
                    payload: queriedToken.payload && JSON.parse(queriedToken.payload)
                };
            }
            else {
                throw new Error('Invalid token');
            }
        };
        this.dynamodb = new AWS.DynamoDB.DocumentClient(config || this.config);
    }
}
exports.dynamoCache = dynamoCache;
//# sourceMappingURL=dynamoCache.js.map