"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.faunaCache = void 0;
// Common
const Cacheable_1 = require("../abstract/Cacheable");
const faunadb = __importStar(require("faunadb"));
// const { log } = require('../../helpers')
class faunaCache extends Cacheable_1.Cacheable {
    constructor() {
        super();
        this.config = { secret: process.env.FAUNADB_SECRET || '' };
        this.q = faunadb.query;
        /**
         * Creates OAuth State
         */
        this.createState = async () => {
            const state = this.hash();
            return this.client.query(this.q.Create(this.q.Class('auth_cache'), {
                data: { token: state, type: 'STATE', expired: false }
            })).then(() => state);
        };
        /**
         * Revokes OAuth State
         * @param state
         */
        this.revokeState = async (state) => this.client.query(this.q.Let({ matched: this.q.Get(this.q.Match(this.q.Index('auth_cache'), state)) }, this.q.If(this.q.And(this.q.Equals(this.q.Select(['data', 'expired'], this.q.Var('matched')), false), this.q.Equals(this.q.Select(['data', 'type'], this.q.Var('matched')), 'STATE')), this.q.Update(this.q.Select('ref', this.q.Var('matched')), { data: { expired: true } }), 'expired'))).then((result) => {
            // log('revokeState', result)
            return state;
        });
        /**
         * Creates and saves refresh token
         * @param user
         */
        this.saveRefreshToken = async (user, payload) => {
            const token = this.hash();
            return this.client.query(this.q.Create(this.q.Class('auth_cache'), {
                data: {
                    token,
                    type: 'REFRESH',
                    expired: false,
                    userId: user,
                    payload: payload || {}
                }
            })).then(() => token);
        };
        /**
         * Revokes old refresh token and creates new
         * @param oldToken
         */
        this.revokeRefreshToken = async (oldToken) => {
            if (!oldToken.match(/[A-Fa-f0-9]{64}/)) {
                return Promise.reject(new Error('Invalid token'));
            }
            const token = this.hash();
            return this.client.query(this.q.Let({ matched: this.q.Get(this.q.Match(this.q.Index('auth_cache'), oldToken)) }, this.q.If(this.q.And(this.q.Equals(this.q.Select(['data', 'expired'], this.q.Var('matched')), false), this.q.Equals(this.q.Select(['data', 'type'], this.q.Var('matched')), 'REFRESH')), this.q.Update(this.q.Select('ref', this.q.Var('matched')), { data: { token } }), 'expired'))).then((result) => {
                // log('revokeRefreshToken', result)
                return { id: result.data.userId, token, payload: result.data.payload };
            });
        };
        this.client = new faunadb.Client(this.config);
    }
}
exports.faunaCache = faunaCache;
//# sourceMappingURL=faunaCache.js.map