"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// call this with `STAGE=dev npm run setup:fauna` before anything else
const faunadb = __importStar(require("faunadb"));
const fs = __importStar(require("fs/promises"));
const yaml = __importStar(require("js-yaml")); // eslint-disable-line import/no-extraneous-dependencies
const readFile = async (filePath) => {
    return await fs.readFile(filePath, 'utf8');
};
const setupSchema = (config, userClassName) => {
    const q = faunadb.query;
    const client = new faunadb.Client(config);
    return client
        .query(q.CreateClass({ name: 'auth_cache' }))
        .then(() => client.query(q.Create(q.Ref('indexes'), {
        name: 'auth_cache',
        source: q.Class('auth_cache'),
        terms: [{ field: ['data', 'token'] }],
        unique: true
    })))
        .then(() => client.query(q.CreateClass({ name: userClassName })))
        .then(() => client.query(q.Create(q.Ref('indexes'), {
        name: 'auth_userId',
        source: q.Class(userClassName),
        terms: [{ field: ['data', 'userId'] }],
        unique: true
    })))
        .then(() => client.query(q.Create(q.Ref('indexes'), {
        // this index is optional but useful in development for browsing users
        name: `all_${userClassName}`,
        source: q.Class(userClassName)
    })));
};
const run = async () => {
    const file = await readFile('./authentication/env.yml');
    const yamlData = yaml.safeLoad(file);
    if (yamlData === undefined || typeof (yamlData) === 'string') {
        throw new Error('yamlData cannot be undefined or a string.');
    }
    const env = yamlData[process.env.STAGE || 'dev'];
    const userClassName = env.USERS_CLASS_NAME || 'users'; // should be shared with content app
    const config = { secret: env.FAUNADB_SECRET };
    process.env.FAUNADB_SECRET = env.FAUNADB_SECRET;
    try {
        await setupSchema(config, userClassName);
    }
    catch (exception) {
        console.error(JSON.stringify(exception, null, 2)); // eslint-disable-line no-console
    }
};
run();
//# sourceMappingURL=faunaSetupSchema.js.map